@@ -6,3 +6,5 @@ Additional `torch.nn.Module`s frequently used for meta-learning.
 
 from .kroneckers import *
 from .misc import *
+from .protonet import PrototypicalClassifier
+from .metaoptnet import SVClassifier
